import { onLoadFinished, onLoadStarted } from "./webview-utils-common";
export class WebViewUtils {
    static addHeaders(wv, headers) {
        WebViewUtils.wv = wv;
        WebViewUtils.headers = headers;
        if (wv.android) {
            wv.android.setWebViewClient(WebViewUtilsExtClient.initWithView(wv));
        }
        else {
            setTimeout(() => {
                wv.android.setWebViewClient(WebViewUtilsExtClient.initWithView(wv));
            });
        }
    }
}
var WebViewUtilsExtClient = /** @class */ (function (_super) {
    __extends(WebViewUtilsExtClient, _super);
    function WebViewUtilsExtClient() {
        var _this = _super.call(this) || this;
        // hackilish flag to make sure we don't fire the onLoadFinished event multiple times
        _this.startEventCount = 0;
        _this.headersAddedTo = new Set();
        return global.__native(_this);
    }
    WebViewUtilsExtClient.initWithView = function (view) {
        var client = new WebViewUtilsExtClient();
        client._view = view;
        client._origClient = view._webViewClient; // getWebViewClient = API level 26 (Android 8)
        return client;
    };
    // Note that this method is overloaded in Java (changed in Lollipop - no longer used, from the looks of it)
    WebViewUtilsExtClient.prototype.shouldOverrideUrlLoading = function (webView, urlOrWebResourceRequest /* string | android.webkit.WebResourceRequest */) {
        var url = typeof urlOrWebResourceRequest === "string"
            ? urlOrWebResourceRequest
            : urlOrWebResourceRequest.getUrl().toString();
        webView.loadUrl(url, this.getAdditionalHeadersForUrl(url));
        return true;
    };
    WebViewUtilsExtClient.prototype.onPageStarted = function (webView, url, favicon) {
        _super.prototype.onPageStarted.call(this, webView, url, favicon);
        var headersAdded = this.headersAddedTo.has(url);
        var isHttpRequest = url.indexOf("http") === 0;
        if (isHttpRequest && !headersAdded) {
            ++this.startEventCount;
            this._view.android.loadUrl(url, this.getAdditionalHeadersForUrl(url));
            if (WebViewUtils.wv) {
                onLoadStarted(WebViewUtils.wv, url, undefined);
            }
        }
        else if (!isHttpRequest || (headersAdded && WebViewUtils.wv)) {
            if (!isHttpRequest || ++this.startEventCount === 1) {
                onLoadStarted(WebViewUtils.wv, url, undefined);
            }
        }
    };
    WebViewUtilsExtClient.prototype.onPageFinished = function (view, url) {
        _super.prototype.onPageFinished.call(this, view, url);
        var isHttpRequest = url.indexOf("http") === 0;
        if (!isHttpRequest || (WebViewUtils.wv && this.startEventCount > 1)) {
            onLoadFinished(WebViewUtils.wv, url, undefined);
        }
    };
    WebViewUtilsExtClient.prototype.onReceivedError = function () {
        var view = arguments[0];
        if (arguments.length === 4) {
            var errorCode = arguments[1];
            var description = arguments[2];
            var failingUrl = arguments[3];
            _super.prototype.onReceivedError.call(this, view, errorCode, description, failingUrl);
            if (WebViewUtils.wv) {
                onLoadFinished(WebViewUtils.wv, failingUrl, description + "(" + errorCode + ")");
            }
        }
        else {
            var request = arguments[1];
            var error = arguments[2];
            _super.prototype.onReceivedError.call(this, view, request, error, undefined);
            var headersAdded = error.getUrl && this.headersAddedTo.has(error.getUrl());
            if (headersAdded && WebViewUtils.wv) {
                onLoadFinished(WebViewUtils.wv, error.getUrl && error.getUrl(), error.getDescription() + "(" + error.getErrorCode() + ")");
            }
        }
    };
    WebViewUtilsExtClient.prototype.getAdditionalHeadersForUrl = function (url) {
        var _this = this;
        var headers = new java.util.HashMap();
        if (!this.headersAddedTo.has(url)) {
            WebViewUtils.headers.forEach(function (val, key) {
                headers.put(key, val);
                if (key.toLowerCase() === "user-agent") {
                    _this._view.android.getSettings().setUserAgentString(val);
                }
            });
            this.headersAddedTo.add(url);
        }
        return headers;
    };
    return WebViewUtilsExtClient;
}(android.webkit.WebViewClient));
