import { onLoadFinished, onLoadStarted } from "./webview-utils-common";
var WebviewUtilsWKNavigationDelegateImpl = /** @class */ (function (_super) {
    __extends(WebviewUtilsWKNavigationDelegateImpl, _super);
    function WebviewUtilsWKNavigationDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WebviewUtilsWKNavigationDelegateImpl.initWithOwnerAndHeaders = function (owner, headers) {
        var handler = WebviewUtilsWKNavigationDelegateImpl.new();
        handler._owner = owner;
        handler.headers = headers;
        return handler;
    };
    WebviewUtilsWKNavigationDelegateImpl.prototype.webViewDecidePolicyForNavigationActionDecisionHandler = function (webView, navigationAction, decisionHandler) {
        var owner = this._owner.get();
        if (!owner || !navigationAction.request.URL) {
            return;
        }
        var isHttpRequest = navigationAction.request.URL.absoluteString.indexOf("http") === 0;
        var areHeadersAdded = true;
        this.headers.forEach(function (val, key) {
            areHeadersAdded = areHeadersAdded && navigationAction.request.valueForHTTPHeaderField(key) === val;
        });
        if (isHttpRequest && !areHeadersAdded) {
            if (navigationAction.request.HTTPMethod !== "GET") {
                decisionHandler(WKNavigationActionPolicy.Allow);
                return;
            }
            decisionHandler(WKNavigationActionPolicy.Cancel);
            var customRequest_1 = navigationAction.request.mutableCopy();
            // add the original headers
            for (var i = 0; i < navigationAction.request.allHTTPHeaderFields.count; i++) {
                var headerKey = navigationAction.request.allHTTPHeaderFields.allKeys.objectAtIndex(i);
                customRequest_1.setValueForHTTPHeaderField(navigationAction.request.allHTTPHeaderFields.objectForKey(headerKey), headerKey);
            }
            this.headers.forEach(function (val, key) {
                customRequest_1.setValueForHTTPHeaderField(val, key);
                // the user agent is set as header, but also set this property:
                if (key.toLowerCase() === "user-agent") {
                    webView.customUserAgent = val;
                }
            });
            webView.loadRequest(customRequest_1);
        }
        else {
            decisionHandler(WKNavigationActionPolicy.Allow);
            var navType = "other";
            switch (navigationAction.navigationType) {
                case WKNavigationType.LinkActivated:
                    navType = "linkClicked";
                    break;
                case WKNavigationType.FormSubmitted:
                    navType = "formSubmitted";
                    break;
                case WKNavigationType.BackForward:
                    navType = "backForward";
                    break;
                case WKNavigationType.Reload:
                    navType = "reload";
                    break;
                case WKNavigationType.FormResubmitted:
                    navType = "formResubmitted";
                    break;
            }
            onLoadStarted(owner, navigationAction.request.URL.absoluteString, navType);
        }
    };
    WebviewUtilsWKNavigationDelegateImpl.prototype.webViewDidFinishNavigation = function (webView, navigation) {
        var owner = this._owner.get();
        if (owner) {
            var src = owner.src;
            if (webView.URL) {
                src = webView.URL.absoluteString;
            }
            onLoadFinished(owner, src);
        }
    };
    WebviewUtilsWKNavigationDelegateImpl.prototype.webViewDidFailNavigationWithError = function (webView, navigation, error) {
        var owner = this._owner.get();
        if (owner) {
            var src = owner.src;
            if (webView.URL) {
                src = webView.URL.absoluteString;
            }
            onLoadFinished(owner, src, error.localizedDescription);
        }
    };
    WebviewUtilsWKNavigationDelegateImpl.ObjCProtocols = [WKNavigationDelegate];
    return WebviewUtilsWKNavigationDelegateImpl;
}(NSObject));
export class WebViewUtils {
    static addHeaders(wv, headers) {
        if (WebViewUtils.isWKWebView(wv)) {
            wv.nativeViewProtected.navigationDelegate = wv._delegate = WebviewUtilsWKNavigationDelegateImpl.initWithOwnerAndHeaders(new WeakRef(wv), headers);
        }
        else {
            wv._delegate = WebViewUtilsDelegateImpl.initWithOwner(new WeakRef(wv));
            WebViewUtils.headers = headers;
        }
    }
    static isWKWebView(wv) {
        return wv.nativeViewProtected.isKindOfClass(WKWebView.class());
    }
}
var WebViewUtilsDelegateImpl = /** @class */ (function (_super) {
    __extends(WebViewUtilsDelegateImpl, _super);
    function WebViewUtilsDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WebViewUtilsDelegateImpl.initWithOwner = function (owner) {
        var delegate = WebViewUtilsDelegateImpl.new();
        delegate._owner = owner;
        delegate._originalDelegate = owner.get()._delegate;
        return delegate;
    };
    // UIWebView delegate
    WebViewUtilsDelegateImpl.prototype.webViewShouldStartLoadWithRequestNavigationType = function (webView, request, navigationType) {
        var urlString = request.URL.absoluteString;
        var isNavigationTypeBackForward = navigationType === UIWebViewNavigationType.BackForward;
        var isHttpRequest = urlString.indexOf("http") === 0;
        var areHeadersAdded = true;
        WebViewUtils.headers.forEach(function (val, key) {
            areHeadersAdded = areHeadersAdded && request.valueForHTTPHeaderField(key) === val;
        });
        if (!isHttpRequest || isNavigationTypeBackForward || areHeadersAdded || urlString === "about:blank") {
            return this._originalDelegate.webViewShouldStartLoadWithRequestNavigationType(webView, request, navigationType);
        }
        var nsMutableURLRequest = new NSMutableURLRequest({
            URL: request.URL,
            cachePolicy: NSURLRequestCachePolicy.UseProtocolCachePolicy,
            timeoutInterval: 60
        });
        // add the original headers
        for (var i = 0; i < request.allHTTPHeaderFields.count; i++) {
            var headerKey = request.allHTTPHeaderFields.allKeys.objectAtIndex(i);
            nsMutableURLRequest.setValueForHTTPHeaderField(request.allHTTPHeaderFields.objectForKey(headerKey), headerKey);
        }
        WebViewUtils.headers.forEach(function (val, key) {
            nsMutableURLRequest.setValueForHTTPHeaderField(val, key);
            if (key.toLowerCase() === "user-agent") {
                // note that this overrides the useragent for ALL webviews for the app, but that's prolly not a problem
                NSUserDefaults.standardUserDefaults.registerDefaults(NSDictionary.dictionaryWithObjectForKey(val, "UserAgent"));
            }
        });
        webView.loadRequest(nsMutableURLRequest);
        return false;
    };
    // UIWebView delegate
    WebViewUtilsDelegateImpl.prototype.webViewDidStartLoad = function (webView) {
        this._originalDelegate.webViewDidStartLoad(webView);
    };
    // UIWebView delegate
    WebViewUtilsDelegateImpl.prototype.webViewDidFinishLoad = function (webView) {
        this._originalDelegate.webViewDidFinishLoad(webView);
    };
    // UIWebView delegate
    WebViewUtilsDelegateImpl.prototype.webViewDidFailLoadWithError = function (webView, error) {
        this._originalDelegate.webViewDidFailLoadWithError(webView, error);
    };
    WebViewUtilsDelegateImpl.ObjCProtocols = [UIWebViewDelegate];
    return WebViewUtilsDelegateImpl;
}(NSObject));
